from random import random

import numpy as np


def cosine_similarity(a, b):
    return np.dot(a, b) / (np.linalg.norm(a) * np.linalg.norm(b))


def hex_to_rgb_float(hex_str):
    hex_str = hex_str.lstrip("#")
    return tuple(int(hex_str[i : i + 2], 16) / 255.0 for i in (0, 2, 4))


def randomly(prob):
    def decorator(function):
        def wrapper(*args, **kwargs):
            if prob > random():
                return function(*args, **kwargs)

        return wrapper

    return decorator


def pythonify_json(json_data):

    correctedDict = {}

    for key, value in json_data.items():
        if isinstance(value, list):
            value = [
                pythonify_json(item) if isinstance(item, dict) else item
                for item in value
            ]
        elif isinstance(value, dict):
            value = pythonify_json(value)
        try:
            key = int(key)
        except Exception:
            pass
        correctedDict[key] = value

    return correctedDict



import random
from typing import Optional


def with_probabilities(
    *probabilities: tuple[tuple],
) -> Optional[any]:
    """
    Takes a tuple of ((function, args), probability) pairs and returns a
    the result of a random function call based on the probabilities.
    Returns None if no function is called.
    """
    choices, probs = zip(*probabilities)
    total_prob = sum(probs)
    if total_prob > 1:
        raise (ValueError("Probabilities sum must be less than or equal to 1"))
    if total_prob < 1:
        choices = choices + (None,)
        probs = probs + (1 - total_prob,)
    res = random.choices(choices, weights=probs)[0]
    return res



import asyncio
from functools import partial

from charset_normalizer import logging

from marsbots.resources.language_models import LanguageModel
from marsbots.resources.language_models import OpenAIGPT3LanguageModel


async def complete_text(
    language_model: LanguageModel,
    prompt: str,
    max_tokens: int,
    use_content_filter: bool = False,
    **kwargs: any,
) -> str:
    logging.info(f"Completing text with prompt: {prompt}")
    loop = asyncio.get_running_loop()
    response_safe, max_tries, num_tries = False, 3, 0
    while num_tries < max_tries and not response_safe:
        completion_text = await loop.run_in_executor(
            None,
            partial(
                language_model.completion_handler,
                prompt=prompt,
                max_tokens=int(max_tokens),
                **kwargs,
            ),
        )
        num_tries += 1
        if (
            OpenAIGPT3LanguageModel.content_safe(completion_text)
            or not use_content_filter
        ):
            response_safe = True
        else:
            print(f"Completion flagged unsafe: {completion_text}")
            logging.info(f"Completion flagged unsafe: {completion_text}")
    if not response_safe:
        completion_text = "Sorry, try talking about something else."
    logging.info(f"Completed text: {completion_text}")
    return completion_text



from dataclasses import dataclass
from typing import List


@dataclass
class MarsBotSettings:
    name: str
    token_env: str
    command_prefix: str
    intents: List[str]


@dataclass
class ChatMessage:
    content: str
    sender: str
    deliniator_left: str = "**["
    deliniator_right: str = "]**:"

    def __str__(self) -> str:
        return (
            f"{self.deliniator_left}{self.sender}{self.deliniator_right} {self.content}"
        )



from abc import ABC
from abc import abstractmethod
from dataclasses import dataclass
from typing import Any
from typing import List

import cohere
import numpy as np
import openai
import requests

from marsbots import config
from marsbots.util import cosine_similarity


class LanguageModel(ABC):
    def __init__(self, model_name: str) -> None:
        self.model_name = model_name

    @abstractmethod
    def completion_handler(self, prompt: str, **kwargs: Any) -> str:
        raise NotImplementedError


@dataclass
class OpenAIGPT3LanguageModelSettings:
    engine: str = "text-davinci-002"
    temperature: float = 1.0
    top_p: float = 1.0
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0


class OpenAIGPT3LanguageModel(LanguageModel):
    def __init__(
        self,
        model_name: str = "openai-gpt3",
        api_key: str = config.LM_OPENAI_API_KEY,
        **kwargs,
    ) -> None:
        self.settings = OpenAIGPT3LanguageModelSettings(**kwargs)
        openai.api_key = api_key
        super().__init__(model_name)

    def completion_handler(
        self,
        prompt: str,
        max_tokens: int,
        stop: list = None,
        **kwargs: any,
    ) -> str:
        completion = openai.Completion.create(
            engine=self.settings.engine,
            prompt=prompt,
            max_tokens=max_tokens,
            stop=stop,
            temperature=kwargs.get("temperature") or self.settings.temperature,
            top_p=kwargs.get("top_p") or self.settings.top_p,
            frequency_penalty=kwargs.get("frequency_penalty")
            or self.settings.frequency_penalty,
            presence_penalty=kwargs.get("presence_penalty")
            or self.settings.presence_penalty,
        )
        completion_text = completion.choices[0].text
        return completion_text

    @staticmethod
    def content_safe(query: str) -> bool:
        # https://beta.openai.com/docs/engines/content-filter
        response = openai.Completion.create(
            engine="content-filter-alpha",
            prompt="<|endoftext|>" + query + "\n--\nLabel:",
            temperature=0,
            max_tokens=1,
            top_p=0,
            logprobs=10,
        )
        output_label = response["choices"][0]["text"]
        toxic_threshold = -0.355
        if output_label == "2":
            logprobs = response["choices"][0]["logprobs"]["top_logprobs"][0]
            if logprobs["2"] < toxic_threshold:
                logprob_0 = logprobs.get("0", None)
                logprob_1 = logprobs.get("1", None)
                if logprob_0 is not None and logprob_1 is not None:
                    if logprob_0 >= logprob_1:
                        output_label = "0"
                    else:
                        output_label = "1"
                elif logprob_0 is not None:
                    output_label = "0"
                elif logprob_1 is not None:
                    output_label = "1"
        if output_label not in ["0", "1", "2"]:
            output_label = "2"
        return output_label != "2"

    def document_search(
        self,
        query: str,
        documents: List[str] = None,
        file=None,
        **kwargs,
    ):
        engine = kwargs.get("engine") or self.settings.engine
        search = openai.Engine(engine).search(
            documents=documents,
            query=query,
            file=file,
        )
        return search

    def document_similarity(self, document: str, query: str, **kwargs):
        engine = kwargs.get("engine") or self.settings.engine
        doc_engine = f"text-search-{engine}-doc-001"
        query_engine = f"text-search-{engine}-query-001"
        document_embedding = self._get_embedding(document, engine=doc_engine)
        query_embedding = self._get_embedding(query, engine=query_engine)
        similarity = cosine_similarity(document_embedding, query_embedding)
        return similarity

    def most_similar_doc_idx(self, document_search_result: dict):
        return np.argmax([d["score"] for d in document_search_result["data"]])

    def _get_embedding(self, text: str, engine: str):
        text = text.replace("\n", " ")
        return openai.Embedding.create(input=[text], engine=engine)["data"][0][
            "embedding"
        ]

    def upload_doc(self, document_path: str, purpose: str = "search"):
        openai.File.create(file=document_path, purpose=purpose)


@dataclass
class AI21JurassicLanguageModelSettings:
    model_type: str = "j1-jumbo"
    temperature: float = 1.0
    top_p: float = 1.0


class AI21JurassicLanguageModel(LanguageModel):
    def __init__(
        self,
        model_name: str = "ai21-jurassic",
        api_key: str = config.LM_AI21_API_KEY,
        **kwargs,
    ) -> None:
        self.settings = AI21JurassicLanguageModelSettings(**kwargs)
        self.api_key = api_key
        super().__init__(model_name)

    @property
    def api_url(self) -> str:
        return f"https://api.ai21.com/studio/v1/{self.settings.model_type}/complete"

    def completion_handler(
        self,
        prompt: str,
        max_tokens: int,
        stop: list = None,
        **kwargs: any,
    ) -> str:
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        payload = {
            "prompt": prompt,
            "maxTokens": max_tokens,
            "temperature": kwargs.get("temperature") or self.settings.temperature,
            "topP": kwargs.get("top_p") or self.settings.top_p,
            "stopSequences": stop if stop else [],
        }
        response = requests.post(self.api_url, json=payload, headers=headers)
        completion = response.json()
        completion_text = completion["completions"][0]["data"]["text"]
        return completion_text


@dataclass
class CohereLanguageModelSettings:
    model_type: str = "large"
    temperature: float = 1.0
    top_p: float = 1.0
    top_k: float = 0


class CohereLanguageModel(LanguageModel):
    def __init__(
        self,
        model_name: str = "cohere",
        api_key: str = config.LM_COHERE_API_KEY,
        **kwargs,
    ):
        self.client = cohere.Client(api_key)
        self.settings = CohereLanguageModelSettings(**kwargs)
        super().__init__(model_name)

    def completion_handler(self, prompt: str, max_tokens: int, **kwargs: any) -> str:
        prediction = self.client.generate(
            prompt=prompt,
            max_tokens=max_tokens,
            model=kwargs.get("model_type") or self.settings.model_type,
            temperature=kwargs.get("temperature") or self.settings.model_type,
            k=kwargs.get("top_k") or self.settings.top_k,
            p=kwargs.get("top_p") or self.settings.top_p,
        )
        completion = prediction.generations[0].text
        return completion


@dataclass
class GooseAILanguageModelSettings:
    engine: str = "gpt-neo-20b"
    temperature: float = 1.0
    top_p: float = 1.0
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0


class GooseAILanguageModel(LanguageModel):
    def __init__(
        self,
        model_name: str = "gooseai",
        api_key: str = config.LM_GOOSEAI_API_KEY,
        **kwargs,
    ) -> None:
        self.settings = GooseAILanguageModelSettings(**kwargs)
        openai.api_key = api_key
        openai.api_base = "https://api.goose.ai/v1"
        super().__init__(model_name)

    def completion_handler(
        self,
        prompt: str,
        max_tokens: int,
        stop: list = None,
        **kwargs: any,
    ) -> str:
        completion = openai.Completion.create(
            engine=self.settings.engine,
            prompt=prompt,
            max_tokens=max_tokens,
            stop=stop,
            temperature=kwargs.get("temperature") or self.settings.temperature,
            top_p=kwargs.get("top_p") or self.settings.top_p,
            frequency_penalty=kwargs.get("frequency_penalty")
            or self.settings.frequency_penalty,
            presence_penalty=kwargs.get("presence_penalty")
            or self.settings.presence_penalty,
        )
        completion_text = completion.choices[0].text
        return completion_text



import os

from dotenv import load_dotenv

load_dotenv()

BOT_TOKEN = os.getenv("BOT_TOKEN")
ENV = "DEV"

LM_OPENAI_API_KEY = os.getenv("LM_OPENAI_API_KEY")
LM_EXAFUNCTION_API_KEY = os.getenv("LM_EXAFUNCTION_API_KEY")
LM_AI21_API_KEY = os.getenv("LM_AI21_API_KEY")
LM_COHERE_API_KEY = os.getenv("LM_COHERE_API_KEY")
LM_GOOSEAI_API_KEY = os.getenv("LM_GOOSEAI_API_KEY")

IFTTT_KEY = os.getenv("IFTTT_KEY")

CRASH_WEBHOOK_URL = os.getenv("CRASH_WEBHOOK_URL")
TEST_GUILD_ID = int(os.getenv("TEST_GUILD_ID")) if os.getenv("TEST_GUILD_ID") else None
TEST_CHANNEL_ID = (
    int(os.getenv("TEST_CHANNEL_ID")) if os.getenv("TEST_CHANNEL_ID") else None
)



import re


def remove_empty_lines(response):
    response = re.sub(r"[\n]+", "\\n", response)
    return response


def include_preface(response, settings):
    if settings.preface:
        response = settings.preface + response
    return response



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['marsbots', 'marsbots.programs', 'marsbots.resources']

package_data = \
{'': ['*']}

install_requires = \
['cohere', 'requests', 'openai', 'python-dotenv', 'py-cord==2.0.0b1']

extras_require = \
{'dev': ['black', 'pytest', 'mypy', 'pre-commit', 'flake8']}

setup(name='marsbots',
      version='0.1.0',
      description='The core framework for cool Martian bots.',
      author=None,
      author_email='Jonathan Miller <jon@jmill.dev>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
